# -*- coding: utf-8 -*-

###########################################################################
## File:        functions_general.py
## Description: This file includes various functions
###########################################################################



import data

if(data.UsePySide == True):
    from PySide.QtCore import *
    from PySide.QtGui import *
    from PySide import QtCore, QtGui    
else:    
    from PyQt4.QtCore import *
    from PyQt4.QtGui import *
    from PyQt4 import QtCore, QtGui

import sys
import os
#import random
import time
from time import strftime
#import math
#import array
import string

import ConfigParser
import os, os.path

import dbus
#from dbus.mainloop.qt import DBusQtMainLoop

#import shlex, subprocess


#import gobject
#gobject.threads_init()
#
#import pygst
#pygst.require("0.10")
#import gst


import filter_data #filter
import records #record data

import i18n #load language file
_ = i18n.language.ugettext




###########################################################################
## Function:    LoadConfig
## Description: This function loads the preset names from the preset names file
##                If the parameter is missing in the file, it uses the default value
## Parameters:  none
## Returns:     none
###########################################################################
def LoadPresetNames():
    global PresetNames
    print "Load preset names"

    PresetNames = ConfigParser.SafeConfigParser()
    try:
        PresetNames.read(data.configFolder+data.PresetNamesFile)
    except: #use default PresetNames
        print "PresetNames file "+data.configFolder+data.PresetNamesFile + " not existing or not compatible"

    try:
        PresetNames.add_section('main')
    except:
        pass

    for i in range(0, 4):
        try:
            tmp=PresetNames.get('main', 'preset'+str(i+1))
            data.preset_names[i] = tmp.decode("utf-8")
        except:
             data.preset_names[i]=""


###########################################################################
## Function:    WritePresetNames
## Description: This function loads the preset names into the preset names file
## Parameters:  none
## Returns:     none
###########################################################################
def WritePresetNames():
    global PresetNames
    print "Write preset names"

    for i in range(0, 4):
        if(data.preset_names[i]!=""): name=str(data.preset_names[i].encode( "utf-8" ))
        else: name=""
        PresetNames.set('main', 'Preset'+str(i+1),  name)


    f=data.configFolder+data.PresetNamesFile
    if(data.debug==True): print "Save preset names ("+f+")"
    if (not os.path.exists(data.configFolder)):
        print "Create configuration folder: ", data.configFolder
        os.mkdir(data.configFolder)
#    try:
    handle = open(f, 'w')
    PresetNames.write(handle)
    handle.close()
    print "Preset names saved"
#    except:
#        print "Failed to write preset names file!"






###########################################################################
## Function:    LoadConfig
## Description: This function loads the data from the config file
##                If the parameter is missing in the config file, it uses
##                the default value from file data.py
## Parameters:  none
## Returns:     none
###########################################################################
def LoadConfig():
    #get configuration
    global config
    if(data.debug==True): print "Configuration file:", data.configFolder+data.ConfigFile
    config = ConfigParser.SafeConfigParser()
    try:
        config.read(data.configFolder+data.ConfigFile)
    except: #use default config
        print "Configuration file "+data.configFolder+data.ConfigFile + " not existing or not compatible"

    try:
        config.add_section('main')
    except:
        pass


    data.Sampling_interval = Config_Load_Int('main', 'sampling', data.Sampling_interval)
    data.Sample_smoothing = Config_Load_Int('main', 'Sample_smoothing', data.Sample_smoothing)
    data.Value_smoothing = Config_Load_Int('main', 'Value_smoothing', data.Value_smoothing)
    data.Trigger = Config_Load_Int('main', 'Trigger', data.Trigger)
    data.Derivation = Config_Load_Int('main', 'Derivation', data.Derivation)
    data.Ysize = Config_Load_Int('main', 'Ysize', data.Ysize)
    data.Xscale = Config_Load_Int('main', 'Xscale', data.Xscale)
    data.Yscale_Movement = Config_Load_Int('main', 'Yscale_Movement', data.Yscale_Movement_default)
    data.derivation_offset = Config_Load_Int('main', 'derivation_offset', data.derivation_offset)
    data.Yscale_Sleep_Pattern = Config_Load_Int('main', 'Yscale_Sleep_Pattern', data.Yscale_Sleep_Pattern_default)
    data.Yoffset_Sleep_Pattern = Config_Load_Int('main', 'Yoffset_Sleep_Pattern', data.Yoffset_Sleep_Pattern_default)
    data.limitMovement = Config_Load_Int('main', 'limitMovement', data.limitMovement)
    data.DelayStartTime = Config_Load_Int('main', 'DelayStartTime', data.DelayStartTime)
    data.AlarmHour = Config_Load_Int('main', 'AlarmHour', data.AlarmHour)
    data.AlarmMinute = Config_Load_Int('main', 'AlarmMinute', data.AlarmMinute)
    data.AlarmTimeWindow = Config_Load_Int('main', 'AlarmTimeWindow', data.AlarmTimeWindow)
    data.Lucid_Dream_Song_Volume = Config_Load_Int('main', 'Lucid_Dream_Song_Volume', data.Lucid_Dream_Song_Volume)
    data.Lucid_Dream_Song_Delay = Config_Load_Int('main', 'Lucid_Dream_Song_Delay', data.Lucid_Dream_Song_Delay)
    data.Lucid_Dream_Song_Duration = Config_Load_Int('main', 'Lucid_Dream_Song_Duration', data.Lucid_Dream_Song_Duration)
    data.WakeUp_Song_Volume_Increase_Step = Config_Load_Int('main', 'WakeUp_Song_Volume_Increase_Step', data.WakeUp_Song_Volume_Increase_Step)
    data.WakeUp_Song_Volume = Config_Load_Int('main', 'WakeUp_Song_Volume', data.WakeUp_Song_Volume)
    data.WakeUp_Song_Duration = Config_Load_Int('main', 'WakeUp_Song_Duration', data.WakeUp_Song_Duration)
    data.Light_Sleep_Level = Config_Load_Int('main', 'Light_Sleep_Level', data.Light_Sleep_Level_default)
    data.Deep_Sleep_Level = Config_Load_Int('main', 'Deep_Sleep_Level', data.Deep_Sleep_Level_default)


#    data.user = Config_Load_String('main', 'user', data.user)
    try:
        tmp=config.get('main', 'user')
        data.user = tmp.decode("utf-8")
    except:
        config.set('main', 'user', str(data.user.encode( "utf-8" )))

    data.LastRecord = Config_Load_String('main', 'LastRecord', data.LastRecord)
    data.AlternativeAlarm = Config_Load_String('main', 'AlternativeAlarm', data.AlternativeAlarm)
    data.AlternativeAlarm = data.AlternativeAlarm.decode("utf-8")
    
    data.Lucid_Dream_Song = Config_Load_String('main', 'Lucid_Dream_Song', data.Lucid_Dream_Song)
    data.Lucid_Dream_Song = data.Lucid_Dream_Song.decode("utf-8")
    data.WakeUp_Song = Config_Load_String('main', 'WakeUp_Song', data.WakeUp_Song)
    data.WakeUp_Song = data.WakeUp_Song.decode("utf-8")
    data.cliStart = Config_Load_String('main', 'cliStart', data.cliStart)
#    data.cliStart = data.cliStart.decode("utf-8") ##has to be solved differently!
    data.cliStop = Config_Load_String('main', 'cliStop', data.cliStop)
#    data.cliStart = data.cliStart.decode("utf-8")  ##has to be solved differently!


    data.OfflineMode = Config_Load_Bool('main', 'OfflineMode', data.OfflineMode)
    data.SilentProfile = Config_Load_Bool('main', 'SilentProfile', data.SilentProfile)
    data.DimmDisplay = Config_Load_Bool('main', 'DimmDisplay', data.DimmDisplay)
    data.UseAlarm = Config_Load_Bool('main', 'UseAlarm', data.UseAlarm)
    data.StopAfterAlarm = Config_Load_Bool('main', 'StopAfterAlarm', data.StopAfterAlarm)
    data.Use_Lucid_Dream_Song = Config_Load_Bool('main', 'Use_Lucid_Dream_Song', data.Use_Lucid_Dream_Song)
    data.Use_WakeUp_Song = Config_Load_Bool('main', 'Use_WakeUp_Song', data.Use_WakeUp_Song)
    data.UseAlternativeAlarm = Config_Load_Bool('main', 'UseAlternativeAlarm', data.UseAlternativeAlarm)




## Add forced configuration changes here
    data.configFileVersion_old = Config_Load_Int('main', 'configFileVersion', 0)
    if(data.configFileVersion_old<data.configFileVersion): #config file is old, update some parameters...
        if(data.configFileVersion_old<=3): #adjust scaling
            data.Yscale_Movement=data.Yscale_Movement_default
            config.set('main', 'Yscale_Movement', str(data.Yscale_Movement_default))

    else: #config file up to date
        pass
        

    print "Configuration successful loaded from "+data.ConfigFile
#    PrintConfig() ##debug only
    return config






###########################################################################
## Function:    Config_Load_String
## Description: Loads a string from the configuration. Uses default value in case econfig ntry does not exist
## Parameters:  section,  entry,  default
## Returns:     value
###########################################################################
def Config_Load_String(section,  entry,  default):
    try:
 #       return (config.get(section, entry)).decode("utf-8")
        return (config.get(section, entry))
    except:
        config.set(section, entry, str(default))
#        config.set(section, entry, str(default.encode( "utf-8" )))
        return str(default)






###########################################################################
## Function:    Config_Load_Bool
## Description: Loads a bool value from the configuration. Uses default value in case econfig ntry does not exist
## Parameters:  section,  entry,  default
## Returns:     value
###########################################################################
def Config_Load_Bool(section,  entry,  default):
    try:
        tmp=config.get(section, entry)
        if(tmp=="True"):  return True
        else: return False
    except:
        config.set(section, entry, str(default))
        return default





###########################################################################
## Function:    Config_Load_Int
## Description: Loads an integer from the configuration. Uses default value in case econfig ntry does not exist
## Parameters:  section,  entry,  default
## Returns:     value
###########################################################################
def Config_Load_Int(section,  entry,  default):
    try:
        return config.getint(section, entry)
    except:
        config.set(section, entry, str(default))
        return int(default)





###########################################################################
## Function:    WriteConfig
## Description: This function loads the config into the config file
## Parameters:  none
## Returns:     none
###########################################################################
def WriteConfig():
    global config
    config.set('main', 'Trigger', str(data.Trigger))
    config.set('main', 'ConfigFileVersion', str(data.configFileVersion))

    f=data.configFolder+data.ConfigFile
    if(data.debug==True): print "Save configuration ("+f+")"
    if (not os.path.exists(data.configFolder)):
        print "Create configuration folder: ", data.configFolder
        try:
            os.mkdir(data.configFolder)
        except: #can not create folder
            print "Can not save the configuration. The folder "+data.configFolder+" can not be created. Are we on a N900?"
            txt="Can not save the configuration. The folder \n.."+data.configFolder[-40:]+"\n can not be created. Are we on a N900?"
            QMessageBox.critical(None, "SleepAnalyser ", txt, QMessageBox.Ok) #Max 11 lines in dialog!
            return #continue application but do not store config
    try:
        handle = open(f, 'w')
        config.write(handle)
        handle.close()
        print "Configuration saved"
    except:
        print "Failed to write configuration file!"




###########################################################################
## Function:    PrintConfig
## Description: This function prints out the current configuration
##                It is for debugging only
## Parameters:  none
## Returns:     none
###########################################################################
def PrintConfig():
    print "###################### CONFIG DATA #######################"
    a=config.items("main")
    for i in range(0, len(a)): #print data
        print a[i]
    print "##########################################################"




###########################################################################
## Function:    Seconds2Tuple
## Description: This function converts the integer value of seconds since 1970
##                into a time tuple
## Parameters:  seconds since 1970
## Returns:     time tuple
###########################################################################
def Seconds2Tuple(secs):
    tuple=time.localtime(secs)
    return tuple




###########################################################################
## Function:    Tuple2Seconds
## Description: This function converts the time tupple into an integer
##                of seconds since 1970
## Parameters:  time tuple
## Returns:     seconds since 1970
###########################################################################
def Tuple2Seconds(Tuple):
    secs=int(time.mktime(Tuple))
    return secs




###########################################################################
## Function:    Tuple2Index
## Description: This function calculates the index in the recod array from
##                the parameter Tuple, relative to the record start time
## Parameters:  time Tuple
## Returns:     record array index
###########################################################################
def Tuple2Index(Tuple):
    t0=records.StartTime_seconds
    t1=Tuple2Seconds(Tuple)
    tdiff=t1-t0
    return int((tdiff-10)*1000/(records.Sampling_interval*records.Value_smoothing))+3





###########################################################################
## Function:    Index2Tuple
## Description: This function calculates the time tuple out of the
##                parameter Index relative to the record start time
## Parameters:  Index from record array
## Returns:     time tuple
###########################################################################
def Index2Tuple(Index):
    tdiff=int(Index*records.Sampling_interval*records.Value_smoothing/1000-3)+10
    t=records.StartTime_seconds+tdiff
    return Seconds2Tuple(t)




###########################################################################
## Function:    set_profile
## Description: This function switches to the requested online/offline mode
## Parameters:  online: True (switch to online mode), False (switch to offline mode)
## Returns:     none
###########################################################################
def set_profile(online):
    if(online==True):
        try:
            if(data.PreviousMode=="normal"): #was online mode (before start measurement), so restore
                print "# Change to online mode"
                cmd="dbus-send --system --dest=com.nokia.mce --type=method_call /com/nokia/mce/request com.nokia.mce.request.req_device_mode_change string:\"normal\""
                os.popen(cmd)
        except:
            pass
    else: #offline
        try:
            cmd="dbus-send --system --dest=com.nokia.mce --print-reply --type=method_call /com/nokia/mce/request com.nokia.mce.request.get_device_mode"
            ret=os.popen(cmd).read() #get current mode
            tmp=string.split(ret, "\"")
            data.PreviousMode=tmp[1]
            print "Mode:", data.PreviousMode
            if(data.PreviousMode=="normal"): #was normal, change to offline
                print "# Change to offline mode"
                cmd="dbus-send --system --dest=com.nokia.mce --type=method_call /com/nokia/mce/request com.nokia.mce.request.req_device_mode_change string:\"offline\""
                os.popen(cmd)
        except:
            pass





###########################################################################
## Function:    get_silent
## Description: This function returs the current profile (general/silent)
## Parameters: none
## Returns:     True (Silent), False (General)
###########################################################################
def get_Silent():
    if(data.DeviceIsN900==True):
        try:
            cmd="dbus-send --type=method_call --print-reply --dest=com.nokia.profiled /com/nokia/profiled com.nokia.profiled.get_profile"
            ret=os.popen(cmd).read() #get current profile
            tmp=string.split(ret, "\"")
            print "Profile:", tmp[1]
            if(tmp[1]=="silent"): return True
            else: return False
        except:
            return False
    else: return False






###########################################################################
## Function:    set_silent
## Description: This function switches to the requested profile
## Parameters:  silent: True (switch to silent profile), False (switch to normal profile)
## Returns:     none
###########################################################################
def set_silent(set_silent):
    if(set_silent==True):
        try:
            if(get_Silent()==False): #was general profile, change to silent profile
                cmd="dbus-send --type=method_call --dest=com.nokia.profiled /com/nokia/profiled com.nokia.profiled.set_profile string:\"silent\" | echo \"\""
                os.popen(cmd)
                print "# Silent Profile activated"
                Notify(_("Silent Profile activated"))
        except:
            pass
    else: #normal
        try:
            if(get_Silent()==True): #was silent (before start measurement), so restore to general
                cmd="dbus-send --type=method_call --dest=com.nokia.profiled /com/nokia/profiled com.nokia.profiled.set_profile string:\"general\" | echo \"\""
                os.popen(cmd)  #set profile
                print "# General Profile activated"
                Notify(_("General Profile activated"))
        except:
            pass




###########################################################################
## Function:    SetVolume
## Description: This function sets the requested volume
## Parameters:  volume percent: 0..100
## Returns:     none
###########################################################################
def SetVolume(percent):
    p=int(percent)
    cmd="dbus-send --type=method_call --dest=com.nokia.mafw.renderer.Mafw-Gst-Renderer-Plugin.gstrenderer /com/nokia/mafw/renderer/gstrenderer com.nokia.mafw.extension.set_extension_property string:volume variant:uint32:"+str(p)
    os.popen(cmd)





###########################################################################
## Function:    GetVolume
## Description: This function returns the requested volume
## Parameters:  none
## Returns:     volume percent: 0..100
###########################################################################
def GetVolume():
    if(data.DeviceIsN900==True): #we are on a N900
        bus = dbus.SessionBus()
        obj = bus.get_object("com.nokia.mafw.renderer.Mafw-Gst-Renderer-Plugin.gstrenderer",
            "/com/nokia/mafw/renderer/gstrenderer")
        mafw = dbus.Interface(obj, "com.nokia.mafw.extension")
        volume = int(mafw.get_extension_property("volume")[1])
    #    print "volume,", volume
        return volume








###########################################################################
## Function:    GetBrightness
## Description: Get display brightness, store it in data.PreviousBrightness
## Parameters:  none
## Returns:     none
###########################################################################
def GetBrightness():
    if(data.DeviceIsN900==True): #we are on a N900
 #       output = os.popen("cat /sys/class/backlight/acx565akm/brightness").read() # run cmd and make the output look like a file I can read
        output = os.popen("gconftool -g /system/osso/dsm/display/display_brightness").read() # run cmd and make the output look like a file I can read
        data.PreviousBrightness=int(output)
        print "Get current brightness level:", data.PreviousBrightness




###########################################################################
## Function:    SetBrightness
## Description: Set display brightness
## Parameters:  brigntness level (1..5)
## Returns:     none
###########################################################################
def SetBrightness(b):
    if(data.DeviceIsN900==True): #we are on a N900
        print "# Set brightness level :", b
 #       cmd="echo "+ str(b)+" > /sys/class/backlight/acx565akm/brightness"
        cmd ="gconftool -s /system/osso/dsm/display/display_brightness -t int "+str(b)
        b = os.popen(cmd)
        b.close()






###########################################################################
## Function:    Notify
## Description: Shows a notification message
## Parameters:  text to be shown
## Returns:     none
###########################################################################
def Notify(notify_txt):
    notify_cmd="dbus-send --type=method_call --dest=org.freedesktop.Notifications /org/freedesktop/Notifications org.freedesktop.Notifications.SystemNoteInfoprint string:\""+notify_txt+"\""
    os.popen(notify_cmd.encode( "utf-8" ))




###########################################################################
## Function:    getFileList
## Description: Returns an array with a file list in a given directory
## Parameters:  directory, file filter
## Returns:     array with files
###########################################################################
def getFileList(path,  filter):
    try:
        path = path.encode("utf-8")
    except:
        pass
        
    cmd = "cd \""+path+"\"; ls -C1 "+str(filter)
    output = os.popen(cmd).read() # run ls and make the output look like a file I can read
    #Note: Above command could also be implemented with os.listdir(). This might give a minor speed improvement
    arr=output.split("\n")
    arr.pop()
    
    print "Check if file/folder does not contain % or ,"
    i=len(arr)-1
    while(i>=0):
#        print i
        if(string.find(arr[i],  "%")!=-1 or string.find(arr[i],  ",")!=-1):
            print "Remove file",  arr[i],  "from list, name contains illegal characters"
            arr.pop(i)
        i=i-1
    
#    print arr
    print len(arr), "files ("+filter+") in", path
    return arr





###########################################################################
## Function:    GetSongLength
## Description: This function returns the duration of a song
## Parameters:  path to song file
## Returns:     song duration in seconds
###########################################################################
def GetSongLength(filename):    
    try:
        filename = filename.encode("utf-8")
    except:
        pass
    try:
        print "get song length for file:", filename
    except:
        pass
    cmd = "ffmpeg -i \""+ filename +"\" 2>&1 | grep Duration"    
    output = os.popen(cmd).read() # run ffmpeg and make the output look like a file I can read
#    print output
    arr=output.split(".")
    arr=arr[0].split(":")

#TODO: Figure out why ffmeg not can return duration on some files
    try:
        duration=int(int(arr[1])*3600+int(arr[2])*60+int(arr[3]))
    except:
        duration=300 #set to 5 minutes
        print "ERROR: Could not get song length!"
        print "Set to default 5 minutes"
#    print duration
    return duration
