# -*- coding: utf-8 -*-

###########################################################################
## File:        functions_record.py
## Description: This file includes the functions needed to modify and read
##                the data in the file record.py
###########################################################################

import sys
import os
import random
import time
from time import strftime
import math
import array
import string

import ConfigParser
import os, os.path

import data
import filter_data #filter
import records #record data

#own functions
from functions_general import *

#own c functions (for speed improvements)
try:
    from lib import mylib
except:
    pass #already handled in main.py


###########################################################################
## Function:    ReadAccSensor
## Description: Reads the acceleration sensor for x times, where
##                x=data.Sample_smoothing (normally 10) and averages it
##                If not on a N900, use randomizer as input
## Parameters:  none
## Returns:     array of accelerations
###########################################################################
def ReadAccSensor():
    coords=[0, 0, 0]
    if(data.DeviceIsN900==True):
        for i in range(data.Sample_smoothing):
            f = open(data.Acceleration_Sensor, 'r' )
            coords_tmp = [int(w) for w in f.readline().split()]
            f.close()
            coords[0]=coords[0] + coords_tmp[0] #X
            coords[1]=coords[1] + coords_tmp[1] #Y
            coords[2]=coords[2] + coords_tmp[2] #Z
        coords[0]=coords[0] / data.Sample_smoothing
        coords[1]=coords[1] / data.Sample_smoothing
        coords[2]=coords[2] / data.Sample_smoothing
    else: #dmo mode, use randomizer
        coords[0]=int(random.uniform(-20,20))
        coords[1]=int(random.uniform(-20,20))
        coords[2]=int(random.uniform(-20,20))
    return coords





###########################################################################
## Function:    GetSensorData
## Description: This function initiates the sensor readings.
##                It gets called once every second from MeasurementTimerLoop()
##                The from ReadAccSensor returned data gets written into a
##                temporary buffer. After x times
##                (where x=data.Value_smoothing, usually 10) it gets averaged
##                again and finally filled into the records array
## Parameters:  none
## Returns:     1 if new data got added to record array, else 0
###########################################################################
def GetSensorData():
    d=ReadAccSensor()
    data.arrDataTemp[0]=data.arrDataTemp[0]+d[0] #X
    data.arrDataTemp[1]=data.arrDataTemp[1]+d[1] #Y
    data.arrDataTemp[2]=data.arrDataTemp[2]+d[2] #Z

    data.sample_index=data.sample_index+1

    if(data.sample_index==data.Value_smoothing): #if is 10 runs over
        data.sample_index=0
#        data.Values_total=data.Values_total+1

        if(records.Index==records.MaxArrayDataLen-1): return 0 #Array full

        records.arrData[0][records.Index]=strftime("%H:%M:%S", time.localtime())
        records.arrData[1][records.Index]=data.arrDataTemp[0]/data.Value_smoothing #X #note: indices are shifted in arrData!
        records.arrData[2][records.Index]=data.arrDataTemp[1]/data.Value_smoothing #Y
        records.arrData[3][records.Index]=data.arrDataTemp[2]/data.Value_smoothing #Z

        records.Index=records.Index+1
        records.LastIndex=max(records.Index, records.LastIndex)

        records.EndTime_seconds=Tuple2Seconds(records.StartTime_tuple)+records.Index*records.Sampling_interval*records.Value_smoothing/1000
        records.EndTime_tuple=Seconds2Tuple(records.EndTime_seconds)

        data.arrDataTemp[0]=0 #X
        data.arrDataTemp[1]=0 #Y
        data.arrDataTemp[2]=0 #Z

        return 1 #added new values to arrData
    else: return 0 #no new data added




###########################################################################
## Function:    GetFirstEntry
## Description: This function fills in the start time and dummy
##                acceleration data, so the record array is not empty
## Parameters:  none
## Returns:     none
###########################################################################
def GetFirstEntry():
    records.arrData[0][0]=strftime("%H:%M:%S", time.localtime())
    records.arrData[1][0]=0 #X
    records.arrData[2][0]=0 #Y
    records.arrData[3][0]=0 #Z

    records.Index=1
    records.LastIndex=max(records.Index, records.LastIndex)





###########################################################################
## Function:    GenerateDerivation
## Description: This function processes the recorded data in the record array
##                The acceleration sensor data (which contains the data of the
##                relative position of the phone) gets derivated into its derivat.
##                This shows spices every time the phone changes the relative position
##                The yellow bars in the graph exactly show this data.
##                If parameter ProcessAll is False, only the last added data
##                gets processed. This saves CPU time
## Parameters:  ProcessAll: True/False
## Returns:     none
###########################################################################
def GenerateDerivation(ProcessAll=False):  #DERIVATION (ABLEITUNG)
    if(data.debug==True): print "Process Data (Process whole array:", ProcessAll, ")"
    data.Progress="Process data..."
    lastmovebeforegraph=0
    if(ProcessAll==True): start=0 #process whole array
    else: start=records.Index-data.Derivation-1 #process only last (changed) records
    if(data.debug==True): print "Process entry", start+data.Derivation, "to", records.Index

    for i in range(start, records.Index-data.Derivation):
        t=i*(records.Sampling_interval*records.Value_smoothing)/1000
        if(data.QuickMode==True or t>data.DelayStartTime):
            Xder=records.arrData[1][i+data.Derivation] - records.arrData[1][i] #X  in 50 sec - X now
            Yder=records.arrData[2][i+data.Derivation] - records.arrData[2][i]  #Y  in 50 sec - Y now
            Zder=records.arrData[3][i+data.Derivation] - records.arrData[3][i]  #Z  in 50 sec - Z now
            XYZderivation=abs(Xder)+abs(Yder)+abs(Zder) #sum of all 3 absolute Derivations
        else: #During delayed phase, do not use data
            Xder=0
            Yder=0
            Zder=0
            XYZderivation=0
        records.arrData[4][i]=abs(XYZderivation-data.derivation_offset)

    #noise reduction of derivation
#    for i in range(data.noiseReduction, records.Index-data.Derivation):
#        val=0
#        for j in range(0, data.noiseReduction):
#            val+=records.arrData[4][i-j]
#        records.arrData[5][i]  = val/data.noiseReduction
#        records.arrData[5][i]  = records.arrData[4][i]

    for i in range(start, records.Index-data.Derivation):
        records.arrData[5][i]  = records.arrData[4][i]





###########################################################################
## Function:    GenerateSleepPattern
## Description: This function calculates the sleep pattern out of the movement
##                data calculated in GenerateDerivation().
##                It multiplies every derivation value with a defined array,
##                which is defined in file filterdata.py
##                As this process is very time consuming, it is exported into
##                a c function. mylib.py contains the Python function, which
##                gets converted into a c function by ShedSkin, a Python to C compiler.
##                If the  c module mylib.so is missing, Python falls back to the
##                slower Python function.
## Parameters:  none
## Returns:     none
###########################################################################
 #Recalculate sleep pattern curve (Note: This function is very time consuming)
def GenerateSleepPattern():
    #NOTE:
    #records.arrData has strings and integers mixed. This ok in python, but does not work in C.
    #So we have to export the data and import it again afterwarts:
    arrSend=[0]*(records.Index+len(filter_data.arrFilter))
    for i in range(0, records.Index-data.Derivation): #copy array
        arrSend[i]=records.arrData[4][i]

    #Exported function:
    arrRet=mylib.CalculateSleepPattern(arrSend, records.Index-data.Derivation, filter_data.arrFilter, filter_data.FilterShift, data.Yscale_Sleep_Pattern, data.Yoffset_Sleep_Pattern, data.Ysize-7, 13, data.limitMovement)

    for i in range(0, records.Index-data.Derivation): #copy array and scale
        records.arrData[6][i]  = arrRet[i]





###########################################################################
## Function:    DrawGraph
## Description: This function draws the record array data into a bitmap
## Parameters:  The target bitmap, QtGui
## Returns:     none
###########################################################################
def DrawGraph(self, QtGui):
    try:
        if(data.debug==True): print "Draw Graph into bitmap"

        Xscale=data.arrXscale[data.Xscale]
        decade=360/Xscale

        paint = QtGui.QPainter()
        paint.begin(self)

        paint.setFont(QtGui.QFont('Decorative', 12))

        #clear whole pitmap (remove old data)
        paint.setPen(QtGui.QColor(data.GraphBackground))
        paint.setBrush(QtGui.QColor(data.GraphBackground))
#        paint.drawRect(0,0,records.MaxArrayDataLen+10,data.GraphBitMapHeight)
        paint.drawRect(0,0, data.GraphBitMapLen, data.Ysize)         
            

        #Draw sleep phase background
        paint.setPen(QtGui.QColor(data.GraphBackground2))
        paint.setBrush(QtGui.QColor(data.GraphBackground2))
        h1=(data.Ysize-(data.Deep_Sleep_Level))
        h2=(data.Ysize-(data.Light_Sleep_Level))
        paint.drawRect(1, h1-6, records.MaxArrayDataLen-2, h2-h1+2)

        #Stop if data is not ok
        if(records.statusID!="OK"):#Show useful infos
            paint.setFont(QtGui.QFont('Decorative', 20))
            paint.setPen(QtGui.QColor("White"))
            paint.drawText(100, data.Ysize/2+5, records.status) #show status information
            return

        #get start data
        GraphStart_tuple=Index2Tuple(0) #time of first record entry
        h=int(strftime("%H", GraphStart_tuple))
        m=int(strftime("%M", GraphStart_tuple))
        s=int(strftime("%S", GraphStart_tuple))

        secondsincefullhour=m*60+s
        GraphStart_seconds=Tuple2Seconds(GraphStart_tuple)
        LastFullHour_sec=GraphStart_seconds-secondsincefullhour
        LastFullHour_tuple=Seconds2Tuple(LastFullHour_sec)

        h0=int(strftime("%H", LastFullHour_tuple))

        LastFullHour_index=Tuple2Index(LastFullHour_tuple)

        GraphStart_index=Tuple2Index(GraphStart_tuple)

        OffsetToLastFullHour=-1*(GraphStart_index - LastFullHour_index)/Xscale
        OffsetToLastFullHour=OffsetToLastFullHour- data.Derivation/Xscale  #Derivation takes 5 indices, so visualisation is delayed!

        #main scaling lines + text
        paint.setPen(QtGui.QColor("white"))
        h=h0-2
        if(data.QuickMode==False): #normal mode
            if(data.Xscale==0): #max zoom
                for i in range(OffsetToLastFullHour-decade, max(data.GraphBitMapLen, records.Index)+decade, decade): #full hours
                    h=(h+1) % 24
                    paint.drawLine(i, data.Ysize, i, data.Ysize+10)
                    paint.setFont(QtGui.QFont('Decorative', 18))
                    paint.drawText(i-25, data.Ysize+32, str(h)+":00")

                    paint.setFont(QtGui.QFont('Decorative', 12))
                    for k in range(1, 6): #hour + k * 10 minutes
                        x=i+k*decade/6
                        paint.drawLine(x, data.Ysize, x, data.Ysize+10)
                        if(x-20>-5): paint.drawText(x-20, data.Ysize+26, str(h)+":"+str(10*k))
                    for m in range(1, 30): #hour + m * 2 minutes
                        x2=i+m*decade/30
                        paint.drawLine(x2, data.Ysize, x2, data.Ysize+4)

            elif(data.Xscale==1 or data.Xscale==2): #mid + min zoom
                for i in range(OffsetToLastFullHour-decade, max(data.GraphBitMapLen, records.Index)+decade, decade): #full hours
                    h=(h+1) % 24
                    paint.drawLine(i, data.Ysize, i, data.Ysize+10)
                    if(i-15>-5): paint.drawText(i-15, data.Ysize+30, str(h)+":00")
                    for k in range(1, 6): #hour + k * 10 minutes
                        x=i+k*decade/6
                        paint.drawLine(x, data.Ysize, x, data.Ysize+4)

        #time bars
        gradient = QtGui.QLinearGradient(0, data.Ysize-11, 0, data.Ysize-1)
        p=0
        p0=0
        while(p<records.Index-data.Derivation):
            if(records.arrData[6][p]>data.Light_Sleep_Level): #awake
                p0=p
                while(records.arrData[6][p]>data.Light_Sleep_Level and p<records.Index-data.Derivation): p=p+1*Xscale
                gradient.setColorAt(0, QtGui.QColor(data.Awake_Color))
                gradient.setColorAt(0.5, QtGui.QColor(data.Awake_ColorDark))
                gradient.setColorAt(1, QtGui.QColor(data.Awake_Color))
                paint.setPen(QtGui.QColor(data.Awake_Color))

            elif(records.arrData[6][p]>data.Deep_Sleep_Level): #light sleep
                p0=p
                while(records.arrData[6][p]>data.Deep_Sleep_Level and records.arrData[6][p]<=data.Light_Sleep_Level and p<records.Index-data.Derivation): p=p+1*Xscale
                gradient.setColorAt(0, QtGui.QColor(data.Light_Sleep_Color))
                gradient.setColorAt(0.5, QtGui.QColor(data.Light_Sleep_ColorDark))
                gradient.setColorAt(1, QtGui.QColor(data.Light_Sleep_Color))
                paint.setPen(QtGui.QColor(data.Light_Sleep_Color))

            else:
                p0=p
                while(records.arrData[6][p]<=data.Deep_Sleep_Level and p<records.Index-data.Derivation): p=p+1*Xscale
                gradient.setColorAt(0, QtGui.QColor(data.Deep_Sleep_Color))
                gradient.setColorAt(0.5, QtGui.QColor(data.Deep_Sleep_ColorDark))
                gradient.setColorAt(1, QtGui.QColor(data.Deep_Sleep_Color))
                paint.setPen(QtGui.QColor(data.Deep_Sleep_Color))

            x0=p0/Xscale
            x1=p/Xscale
            paint.setBrush(QtGui.QBrush(gradient))
            paint.drawRect(x0+1-2, data.Ysize-11, x1-x0-1, 9)

        #movement bars (yellow bars)
        paint.setPen(QtGui.QColor(data.MovementBarColor))
        for p in range(0, records.Index-data.Derivation, Xscale):
            i=p-GraphStart_index
            peak=0
            for j in range(0, Xscale):
                if(records.arrData[5][p+j]>peak): peak=records.arrData[5][p+j]
            x=i/Xscale
            if(peak==""): peak=0 #temporary bugfix
            h=min(peak*data.Yscale_Movement/40, data.Ysize-17) #cut off if higher than graph border
            paint.drawLine(x, data.Ysize-h-1-15, x, data.Ysize-1-15)  #draw yellow movement bar

        #sleep graph (blue graph)
        pen=QtGui.QPen()
        pen.setColor(QtGui.QColor(19, 54, 255, 150))
        pen.setWidth(1)
        paint.setPen(pen)
        for p in range(0, records.Index-data.Derivation-Xscale, Xscale): #sleep pattern filling
            i=p-GraphStart_index
            x=i/Xscale+1
            y=data.Ysize-records.arrData[6][p+Xscale]-5
            paint.drawLine(x, data.Ysize-16, x, y)

        pen.setColor(QtGui.QColor(19, 54, 255))
        pen.setWidth(3) #3
        paint.setPen(pen)
        for p in range(0, records.Index-data.Derivation-Xscale, Xscale): #sleep pattern outline
            i=p-GraphStart_index
            x=i/Xscale+1
            y0=data.Ysize-records.arrData[6][p]-5
            y1=data.Ysize-records.arrData[6][p+Xscale]-5
            paint.drawLine(x, y0, x+1, y1)

        #Lucid dream option
        if(records.Use_Lucid_Dream_Song==True and records.Lucid_Dream_Song_Start_tuple!=-1):
            paint.setBrush(QtGui.QColor(data.Lucid_Dream_Color))
            paint.setPen(QtGui.QColor(data.Lucid_Dream_Color))
            x0=Tuple2Index(records.Lucid_Dream_Song_Start_tuple)-data.Derivation*Xscale
            x00=(x0-GraphStart_index)/Xscale
            t=Seconds2Tuple(Tuple2Seconds(records.Lucid_Dream_Song_Start_tuple)+(records.Lucid_Dream_Song_Duration))
            if(records.AlarmTime_tuple!=-1): end=records.AlarmTime_tuple
            elif(records.AlarmTimeWindowEnd_tuple!=-1): end=records.AlarmTimeWindowEnd_tuple
            else: end=t
            x1=min(Tuple2Index(t), Tuple2Index(end))-data.Derivation*Xscale
            x01=(x1-GraphStart_index)/Xscale
            y=20
#            print "Lucid dream"
            points = [QPointF(x00, y-6), QPointF(x00, y+6), QPointF(x00+6, y)]
            paint.drawPolygon(points)
            paint.drawRect(x00, y-1, x01-x00, 2)

       #WakeUp option
        if(records.Use_WakeUp_Song==True and records.WakeUp_Song_Start_tuple!=-1):
            paint.setBrush(QtGui.QColor(data.WakeUp_Color))
            paint.setPen(QtGui.QColor(data.WakeUp_Color))
            x0=Tuple2Index(records.WakeUp_Song_Start_tuple)-data.Derivation*Xscale
            x00=(x0-GraphStart_index)/Xscale
            t=Seconds2Tuple(Tuple2Seconds(records.WakeUp_Song_Start_tuple)+(records.WakeUp_Song_Duration))
            x1=min(Tuple2Index(t), Tuple2Index(records.AlarmTimeWindowEnd_tuple))-data.Derivation*Xscale
            x01=(x1-GraphStart_index)/Xscale
            y=20
#            print "wake up"
            points=[QPointF(x00, y-6), QPointF(x00, y+6), QPointF(x00+6, y)]
            paint.drawPolygon(points)
            paint.drawRect(x00, y-1, x01-x00, 2)
#            paint.drawPolygon(QPointF(x00, y-5), QPointF(x01, y-5), QPointF(x01, y-8))


       #alarm visualisation
        if(records.AlarmIsSet==True): #alarm time window is set, show it
            paint.setBrush(QtGui.QColor(255, 128, 0, 200))
            paint.setPen(QtGui.QColor(255, 128, 0, 200))
            x0=Tuple2Index(records.AlarmTimeWindowStart_tuple)-data.Derivation*Xscale
            x1=Tuple2Index(records.AlarmTimeWindowEnd_tuple)-data.Derivation*Xscale
            x00=(x0-GraphStart_index)/Xscale
            x10=(x1-GraphStart_index)/Xscale
            paint.drawRect(x00, 8, x10-x00, 2) #Time Window
            paint.drawRect(x10, 4, 1, 10)  #Last Alarm opportunity

            #alarm point
            if(data.AlarmIsActive==True or data.run==False): #only show alarm if alarm occured or in view mode
                if(records.AlarmTime_seconds!=-1):
                    if(records.EndTime_seconds>=records.AlarmTime_seconds-60): #alarm got set off during recording (-60 is workaround)
                        paint.setPen(QtGui.QColor(178, 0, 0))
                        if(records.AlarmTimeWindowEnd_seconds<=records.AlarmTime_seconds): #Alarm due end of alarm time window
                            a0=Tuple2Index(records.AlarmTime_tuple)-data.Derivation*Xscale
                        else: #alarm due movement
                            a0=Tuple2Index(records.AlarmTime_tuple)-data.Derivation
                        a00=(a0-GraphStart_index)/Xscale

                        gradient = QtGui.QRadialGradient(a00, 20, 3)
                        gradient.setColorAt(0, QtGui.QColor(255, 152, 152))
                        gradient.setColorAt(1, QtGui.QColor(255, 0, 0))
                        paint.setBrush(QtGui.QBrush(gradient))
                        paint.drawEllipse(a00-5, 15, 10, 10)

        #DRAW Trigger line
        if(records.AlarmIsSet==True or data.QuickMode==True): #alarm time window is set, show it
            if(records.statusID=="OK"):
                if(data.run==False):  #only show trigger level stored in record when recording is finished
                    t=records.Trigger
                else:
                    t=data.Trigger
                y=(data.Ysize-(t*data.Yscale_Sleep_Pattern*3/40))+10
                paint.setPen(QtGui.QColor(255, 255, 255, 200))
#                paint.drawLine(1, y-17, data.GraphBitMapLen, y-17)  #Trigger line
                if(data.QuickMode==False): x0=Tuple2Index(records.AlarmTimeWindowStart_tuple)-data.Derivation*Xscale
                else: x0=0
                x00=(x0-GraphStart_index)/Xscale
                x00=min(x00, data.GraphBitMapLen-40)
                paint.drawLine(x00, y-17, data.GraphBitMapLen, y-17)  #Trigger line
                if(y<18):
                    y=33
                    #draw arrow
                    xa=data.GraphBitMapLen-37
                    ya=y-21
                    paint.drawLine(xa, ya, xa, ya-9) #arrow
                    paint.drawLine(xa, ya-9, xa-3, ya-6) #arrow
                    paint.drawLine(xa, ya-9, xa+3, ya-6) #arrow
                elif(y<33):
                    y=y+17
                paint.drawText(data.GraphBitMapLen-31, y-19, str(t)) #Trigger level

##        #show parameters (for debugging)
##        paint.setFont(QtGui.QFont('Decorative', 8))
##        paint.setPen(QtGui.QColor("gray"))
##        paint.drawText(data.GraphBitMapLen-70, data.Ysize+12, str(data.Yscale_Movement)+", "+str(data.Yscale_Sleep_Pattern)+", "+str(data.Yoffset_Sleep_Pattern)) #settings (for debbigging)

        

        #When in test mode, make noise if last moveent is over trigger level
        if(data.QuickMode==True): MakeNoise_if_over_trigger(records.arrData[6][records.Index-data.Derivation-1])

    except:
        print "Error while drawing the graph"
        print sys.exc_info()
        
    try:
        #Draw border around graph
        paint.setPen(QtGui.QColor("white"))
        paint.drawLine(0, 0, 0, data.Ysize)  #left side
        paint.drawLine(data.GraphBitMapLen-1, 0, data.GraphBitMapLen-1, data.Ysize)  #right side
        paint.drawLine(0, 0, data.GraphBitMapLen-1, 0)  #top
        paint.drawLine(0, data.Ysize, data.GraphBitMapLen-1, data.Ysize)  #bottom
    except:
        pass

    paint.end()





###########################################################################
## Function:    MakeNoise_if_over_trigger
## Description: This function plays a wav file when ever the movement is
##                over the trigger level.
##                It is only used in quick mode measurement
## Parameters:  current movement
## Returns:     none
###########################################################################
#TODO: improve function!!!!
def MakeNoise_if_over_trigger(peak):
    #beep in test mode
    if(peak>(data.Trigger*data.Yscale_Sleep_Pattern*3/40)): #Trigger Alarm
        if((records.Index-data.Derivation)>data.last_beep): #quick mode for testing but only once per move
            if(data.debug==True): print "Beep"
            data.last_beep=records.Index
            if(data.DeviceIsN900==True): #make beep for testing movement
                b = os.popen("pacat --channels=2 /usr/share/sounds/ui-default_beep.wav")
                b.close()
            print "Beep"





###########################################################################
## Function:    DrawStatistic
## Description: This function draws the statistics into a bitmap
## Parameters:  The target bitmap, QtGui
## Returns:     none
###########################################################################
def DrawStatistic(self, QtGui, QtCore):
    paint = QtGui.QPainter()
    paint.begin(self)

    #clear whole pitmap (remove old data)
    paint.setPen(QtGui.QColor(data.GraphBackground))
    paint.setBrush(QtGui.QColor(data.GraphBackground))
#    paint.drawRect(0, 0, data.StatisticsBitMapLen, data.StatisticsBitMapHeight)


    paint.setPen(QtGui.QColor(data.GraphBackground))

    paint.setFont(QtGui.QFont('Decorative', 12))

    y=10
    paint.setPen(QtGui.QColor("White"))
    paint.drawText(0, y+11, str(records.arrStatData[0][3])+"%")
    paint.drawText(42, y+11, strftime("%Hh %Mm ", time.gmtime(records.arrStatData[0][0])))
    if(records.arrStatData[0][3]>0): #Awake
        gradient = QtGui.QLinearGradient(0, y, 0, y+10)
        gradient.setColorAt(0, QtGui.QColor(data.Awake_Color))
        gradient.setColorAt(0.5, QtGui.QColor(data.Awake_ColorDark))
        gradient.setColorAt(1, QtGui.QColor(data.Awake_Color))
        paint.setBrush(QtGui.QBrush(gradient))
        paint.setPen(QtGui.QColor(data.Awake_Color))
        paint.drawRect(120, y, records.arrStatData[0][3], 10)

    y=30
    paint.setPen(QtGui.QColor("White"))
    paint.drawText(0, y+11, str(records.arrStatData[1][3])+"%")
    paint.drawText(42, y+11, strftime("%Hh %Mm ", time.gmtime(records.arrStatData[1][0])))
    if(records.arrStatData[1][3]>0): #Light Sleep
        gradient = QtGui.QLinearGradient(0, y, 0, y+10)
        gradient.setColorAt(0, QtGui.QColor(data.Light_Sleep_Color))
        gradient.setColorAt(0.5, QtGui.QColor(data.Light_Sleep_ColorDark))
        gradient.setColorAt(1, QtGui.QColor(data.Light_Sleep_Color))
        paint.setBrush(QtGui.QBrush(gradient))
        paint.setPen(QtGui.QColor(data.Light_Sleep_Color))
        paint.drawRect(120, y, records.arrStatData[1][3], 10)

    y=50
    paint.setPen(QtGui.QColor("White"))
    paint.drawText(0, y+11, str(records.arrStatData[2][3])+"%")
    paint.drawText(42, y+11, strftime("%Hh %Mm ", time.gmtime(records.arrStatData[2][0])))
    if(records.arrStatData[2][3]>0): #deep sleep
        gradient = QtGui.QLinearGradient(0, y, 0, y+10)
        gradient.setColorAt(0, QtGui.QColor(data.Deep_Sleep_Color))
        gradient.setColorAt(0.5, QtGui.QColor(data.Deep_Sleep_ColorDark))
        gradient.setColorAt(1, QtGui.QColor(data.Deep_Sleep_Color))
        paint.setBrush(QtGui.QBrush(gradient))
        paint.setPen(QtGui.QColor(data.Deep_Sleep_Color))
        paint.drawRect(120, y, records.arrStatData[2][3], 10)

    paint.end()






###########################################################################
## Function:    GenerateStatisticData
## Description: This function generates the statistics from the record
##                data and puts it into the statistics array
##                As this function also is time consuming, it is exported
##                into a c function. See GenerateSleepPattern() for mre details
## Parameters:  none
## Returns:     none
###########################################################################
def GenerateStatisticData():
    #NOTE:
    #records.arrData has strings and integers mixed. This ok in python, but does not work in C.
    #So we have to export the data and import it again afterwarts:
    arrSend=[0]*(records.Index+len(filter_data.arrFilter))
    for i in range(0, records.Index-data.Derivation): #copy array
        arrSend[i]=records.arrData[6][i]

    records.arrStatData=mylib.CalculateStatistic(arrSend, records.Index-data.Derivation, records.EndTime_seconds-records.StartTime_seconds, records.Sampling_interval*records.Value_smoothing/1000, data.Light_Sleep_Level, data.Deep_Sleep_Level)

    if(data.debug==True): print "Statistic data:", records.arrStatData





###########################################################################
## Function:    InitRecord
## Description: This function clears all data in the record and initialises
##                it for the measurement
## Parameters:  none
## Returns:     none
###########################################################################
def InitRecord():
    records.Index=0
    records.LastIndex=0
    records.StartTime_tuple=time.localtime()
#    records.StartTime_tuple=data.StartTime_seconds
    records.StartTime_seconds=Tuple2Seconds(records.StartTime_tuple)
    records.EndTime_tuple=records.StartTime_tuple
    records.EndTime_seconds=records.StartTime_seconds
    records.AlarmIsSet=False
    records.AlarmTime_tuple=-1
    records.AlarmTime_seconds=-1
    records.AlarmTimeWindowStart_tuple=-1
    records.AlarmTimeWindowStart_seconds=-1
    records.AlarmTimeWindowEnd_tuple=-1
    records.AlarmTimeWindowEnd_seconds=-1
    records.AlarmTimeWindow_minutes=-1
    records.Note=""
    records.Sampling_interval=-1
    records.Sample_smoothing=-1
    records.Value_smoothing=-1
    records.Trigger=-1
    records.status=""
    records.statusID=""

    records.Lucid_Dream_Song_Start_tuple = -1
    records.Use_Lucid_Dream_Song = False
    records.Lucid_Dream_Song = ""
    records.Lucid_Dream_Song_Delay = -1
    records.Lucid_Dream_Song_Volume = -1

    records.WakeUp_Song_Start_tuple = -1
    records.Use_WakeUp_Song = False
    records.WakeUp_Song = ""
    records.WakeUp_Song_Duration = -1
    records.WakeUp_Song_Volume= -1

    for i in range(0, records.MaxArrayDataLen-1): #clear data of old records
        for j in range(0, 6):
            records.arrData[j][i]=0
#        records.arrData[6][i]=0
#        records.arrData[5][i]=""

    print "Reset/cleared all record data"






###########################################################################
## Function:    PrintRecordData
## Description: This function prints out all data in the record.
##                It is only used for debugging
## Parameters:  ShowArray: True/False
## Returns:     none
###########################################################################
def PrintRecordData(ShowArray=True):
    print "###################### RECORD DATA #######################"
    print "Index:", records.Index
    print "LastIndex:", records.LastIndex
    print "StartTime_tuple:", records.StartTime_tuple
    print "StartTime_seconds:", records.StartTime_seconds
    print "EndTime_tuple:", records.EndTime_tuple
    print "EndTime_seconds:", records.EndTime_seconds
    print "AlarmIsSet:", records.AlarmIsSet
    print "AlarmTime_tuple:", records.AlarmTime_tuple
    print "AlarmTime_seconds:", records.AlarmTime_seconds
    print "AlarmTimeWindowStart_tuple:", records.AlarmTimeWindowStart_tuple
    print "AlarmTimeWindowStart_seconds:", records.AlarmTimeWindowStart_seconds
    print "AlarmTimeWindowEnd_tuple:", records.AlarmTimeWindowEnd_tuple
    print "AlarmTimeWindowEnd_seconds", records.AlarmTimeWindowEnd_seconds
    print "AlarmTimeWindow_minutes:", records.AlarmTimeWindow_minutes
    print "Note:", records.Note
    print "Sampling_interval:", records.Sampling_interval
    print "Sample_smoothing:", records.Sample_smoothing
    print "Value_smoothing:", records.Value_smoothing
    print "Trigger:", records.Trigger
    print "Status:", records.statusID, records.status

    print "Use_Lucid_Dream_Song:", records.Use_Lucid_Dream_Song
    print "Lucid_Dream_Song_Start_tuple:", records.Lucid_Dream_Song_Start_tuple
    try:
        f=records.Lucid_Dream_Song.encode( "utf-8" )
    except:
        f=records.Lucid_Dream_Song
    print "Lucid_Dream_Song:", f
    print "Lucid_Dream_Song_Delay:", records.Lucid_Dream_Song_Delay
    print "Lucid_Dream_Song_Duration:", records.Lucid_Dream_Song_Duration
    print "Lucid_Dream_Song_Volume:", records.Lucid_Dream_Song_Volume

    print "Use_WakeUp_Song:", records.Use_WakeUp_Song
    print "WakeUp_Song_Start_tuple:", records.WakeUp_Song_Start_tuple
    try:
        f=records.WakeUp_Song.encode( "utf-8" )
    except:
        f=records.WakeUp_Song
    print "WakeUp_Song:", f
    print "WakeUp_Song_Duration:", records.WakeUp_Song_Duration
    print "WakeUp_Song_Volume:", records.WakeUp_Song_Volume

    print "UseAlternativeAlarm:", records.UseAlternativeAlarm
    print "AlternativeAlarm:", records.AlternativeAlarm

    if(ShowArray==True):
        for i in range(0, records.Index): #print data of  records
            print records.arrData[0][i], records.arrData[1][i], records.arrData[2][i], records.arrData[3][i], records.arrData[4][i], records.arrData[5][i], records.arrData[6][i]

    print "##########################################################"
